package com.example.tomcat;

import org.apache.catalina.connector.Request;
import org.apache.catalina.connector.Response;
import org.apache.catalina.valves.ValveBase;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.Cookie;

public class SharedSessionRequestValve extends ValveBase {

    @Override
    public void invoke(Request request, Response response) throws IOException,
	    ServletException {
	Cookie sessionCookie = getCookie(JSESSIONID, request.getCookies());
	Cookie globalCookie = getCookie(GLOBALID, request.getCookies());

	System.out.println(String.format("%s  invoke()%n   JSESSIONID is %s%n   GLOBALID is %s",
    			  this, cookieToString(sessionCookie), cookieToString(globalCookie)));

	// If the global ID is set, then pass that ID to the session manager
	if ( globalCookie != null )
	{
	    if ( ! SharedSessionManager.setGlobalIdForRequest(globalCookie.getValue()) )
	    {
		// The GLOBALID is not valid so throw it away
		globalCookie = null;
		System.out.println(">>> Throwing out a bad global ID." );
	    }
	    else
	    {
		System.out.println(">>> Passing global ID to session manager." );
	    }
	}
	
	getNext().invoke(request, response);
	
	// Retrieve the container-assigned session ID (null, if already set)
	Cookie responseSessionCookie = getCookie(JSESSIONID, response.getCookies());
	
	// If no global id is set, then record the mapping between the container session
	// and the global ID
	if ( responseSessionCookie != null )
    	{
	    if ( globalCookie == null )
	    {
		// Put the GLOBALID cookie onto the response
		String globalID = responseSessionCookie.getValue();
		globalCookie = new Cookie(GLOBALID, globalID);
		globalCookie.setPath("/");
		if ( responseSessionCookie.getDomain() != null )
		    globalCookie.setDomain(responseSessionCookie.getDomain());
		globalCookie.setMaxAge(responseSessionCookie.getMaxAge());
		globalCookie.setComment(responseSessionCookie.getComment());
		response.addCookie(globalCookie);
		System.out.println(">>> Setting the GLOBALID to " + cookieToString(globalCookie) );
		    
		// Associate the GLOBALID with the session object
		SharedSessionManager.storeGlobalSession(globalID, request.getSessionInternal());
		System.out.println(">>> Associating globalID with " + request.getSessionInternal() );
	    }
    	}
	
	// Print a blank line to separate each reqest in the catalina.out log file
	System.out.println();
    }
    
    private static final String JSESSIONID = "JSESSIONID";
    private static final String GLOBALID = "GLOBALID";
    
    private Cookie getCookie(String name, Cookie[] cookies) {
	if ( cookies == null ) return null;
        for (int i = 0; i < cookies.length; i++) {
            if ( cookies[i].getName().equals(name) ) return cookies[i];
        }
        return null;
    }
    
    private String cookieToString(Cookie cookie)
    {
	if ( cookie == null ) return "";
	
	return "Cookie<name=" + cookie.getName()
	+ " value=" + cookie.getValue()
	+ " domain=" + cookie.getDomain()
	+ " path=" + cookie.getPath()
	+ " maxAge=" + cookie.getMaxAge()
	+ " comment=" + cookie.getComment()
	+ ">";
    }

}
